import pygame
from settings import *


def render_pixarray(sandbox, screen):

	pixarray = pygame.PixelArray(screen)

	for pos in set(sandbox.changedempty):  # update anywhere where elements moved
	#for pos in self.changedempty:  # update anywhere where elements moved
		x0 = BORDER[3] + PIXELSIZE*pos[0]
		y0 = BORDER[0] + PIXELSIZE*pos[1]
		pixarray[x0:x0+PIXELSIZE,y0:y0+PIXELSIZE] = BACKGROUND

	#for pos in set(self.changed):  # update anywhere where elements moved
	for pos in sandbox.changed:  # update anywhere where elements moved
		x0 = BORDER[3] + PIXELSIZE*pos[0]
		y0 = BORDER[0] + PIXELSIZE*pos[1]
		if sandbox.grid[pos[0]][pos[1]] is not None:
			clr = sandbox.grid[pos[0]][pos[1]].color()
		else:
			clr = BACKGROUND
		pixarray[x0:x0+PIXELSIZE, y0:y0+PIXELSIZE] = clr

	del(pixarray) # have to delete the pixelarray to get it to release the lock on the screen so that we can draw the gui.

def render_pixarray_scale(sandbox, surface, screen):
	#srf.fill(BACKGROUND)

	pixarray = pygame.PixelArray(surface)
	#pixarray = pygame.surfarray.pixels2d(srf)

	for pos in set(sandbox.changedempty):  # update anywhere where elements moved
		#for pos in self.changedempty:  # update anywhere where elements moved
		x0 = pos[0]
		y0 = pos[1]
		pixarray[x0,y0] = BACKGROUND

	#for pos in set(self.changed):  # update anywhere where elements moved
	for pos in sandbox.changed:  # update anywhere where elements moved
		x0 = pos[0]
		y0 = pos[1]
		if sandbox.grid[pos[0]][pos[1]] is not None:
			clr = sandbox.grid[pos[0]][pos[1]].color()
		else:
			clr = BACKGROUND
		pixarray[x0, y0] = clr
	#srf3 = pixarray.make_surface()
	del(pixarray) # have to delete the pixelarray to get it to release the lock on the screen so that we can draw the gui.
	#srf2 = pygame.transform.scale(srf, (PIXELSIZE*sandbox.width, PIXELSIZE*sandbox.height))
	#print srf2.get_size()
	#surface.blit(srf2, (BORDER[3], BORDER[0]))
	#srf.blit(surface,  (BORDER[3], BORDER[0]))
	#surface.blit(srf, (BORDER[3], BORDER[0]))
	srf2 = pygame.transform.scale(surface, (PIXELSIZE*sandbox.width, PIXELSIZE*sandbox.height))
	screen.blit(srf2, (BORDER[3],BORDER[0]))

	return surface


class RenderWithGrid(object):
	def __init__(self, sandbox, screen):
		self.sandbox = sandbox
		self.screen = screen
		self.surface = pygame.Surface((sandbox.width, sandbox.height))
		self.gridcolor = (50, 50, 50)
		self.cursorcolor = (90, 90, 90)
		self.show_grid = False
		self.show_cursor = False
		self.cursorsize = 1
		self.cursor = None
		self.change_cursor_size()

	def change_cursor_size(self, size=1):
		self.cursorsize = size
		self.cursor = pygame.Surface((self.cursorsize * PIXELSIZE, self.cursorsize * PIXELSIZE))
		self.cursor.fill(self.cursorcolor)
		self.cursor.set_alpha(90)

	def add_cursor(self, srf):
		x0, y0 = pygame.mouse.get_pos() # first steps are to back out the grid position from the screen pos
		x1 = min(max(0, int((x0-BORDER[3])/PIXELSIZE)), self.sandbox.width)
		y1 = min(max(0, int((y0-BORDER[0])/PIXELSIZE)), self.sandbox.height)
		srf.blit(self.cursor, (x1 * PIXELSIZE, y1 * PIXELSIZE))


	def add_grid(self, srf):
		for x in range(self.sandbox.width):
				pygame.draw.line(srf, self.gridcolor, (PIXELSIZE * x, 0), (PIXELSIZE * x, self.sandbox.height*PIXELSIZE), 1)
		for y in range(self.sandbox.height):
			pygame.draw.line(srf, self.gridcolor, (0, PIXELSIZE * y), (self.sandbox.width*PIXELSIZE, PIXELSIZE * y), 1)

	def render(self):
		#srf.fill(BACKGROUND)

		pixarray = pygame.PixelArray(self.surface)
		#pixarray = pygame.surfarray.pixels2d(srf)

		for pos in set(self.sandbox.changedempty):  # update anywhere where elements moved
			#for pos in self.changedempty:  # update anywhere where elements moved
			x0 = pos[0]
			y0 = pos[1]
			pixarray[x0,y0] = BACKGROUND

		#for pos in set(self.changed):  # update anywhere where elements moved
		for pos in self.sandbox.changed:  # update anywhere where elements moved
			x0 = pos[0]
			y0 = pos[1]
			if self.sandbox.grid[pos[0]][pos[1]] is not None:
				clr = self.sandbox.grid[pos[0]][pos[1]].color()
			else:
				clr = BACKGROUND
			pixarray[x0, y0] = clr
		del(pixarray) # have to delete the pixelarray to get it to release the lock on the screen so that we can draw the gui.
		srf2 = pygame.transform.scale(self.surface, (PIXELSIZE*self.sandbox.width, PIXELSIZE*self.sandbox.height))
		if self.show_cursor:
			self.add_cursor(srf2)
		if self.show_grid:
			self.add_grid(srf2)
		self.screen.blit(srf2, (BORDER[3],BORDER[0]))
